function [ V, E ] = ElectDipole( x,y, p1,p2,q )
%ElectDipole.m 電気双極子の計算
% x-y 平面上にある位置p1(x1,y1) にq, 位置p2(x2,y2) に-q の電荷を
% 置いたときの電位ポテンシャルV を計算し勾配E をグラフ表示する．
% ただし，格子点の間隔は等間隔を想定している．
% 書式
% [ V, E ] = ElectDipole( x,y, p1,p2,q )
% 引数
% x,y：x-y 平面領域　x = xa:xb,y = ya:yb
% p1,p2：電荷位置 p1(x1,y1),p2(x2,y2)
% q：電荷 p1 にq,p2 に-q
% 戻り値
% V：電位ポテンシャル
% E：電位勾配
    Vlimit = 10000; % 特異点の限界値
    k = 9e9; % k = 1/(4 πε 0)
    [X,Y] = meshgrid(x,y);
    dx = x(2)-x(1); dy = y(2)-y(1);
    rp = sqrt((X-p1(1)).^2+(Y-p1(2)).^2); %+q の電位ベクトル
    rm = sqrt((X-p2(1)).^2+(Y-p2(2)).^2); %-q の電位ベクトル
    V = q*k*(1./rp - 1./rm);
    % 特異点の検出と対処
    i = find(V > Vlimit); V(i) = Vlimit*ones(length(i),1);
    i = find(V <-Vlimit); V(i) = -Vlimit*ones(length(i),1);
    [Ex,Ey] = gradient(-V,dx,dy); % 勾配
    E = sqrt(Ex.^2+Ey.^2+eps);
    % 各成分の正規化
    Ex = Ex./E; Ey = Ey./E;
    figure; surfc(X,Y,V);
    xlabel('x'); ylabel('y'); zlabel('Potential V'); 
    title('The fileld and the potential of a dipole');
    hold on
    plot3(p1(1),p1(2),-Vlimit,'o');plot3(p1(1),p1(2),-Vlimit,'+');
    plot3(p2(1),p2(2),-Vlimit,'o');plot3(p2(1),p2(2),-Vlimit,'-');
    Z = -Vlimit*ones(size(Ex)); Ez = zeros(size(Ex));
    quiver3(X(1:2:end), Y(1:2:end), Z(1:2:end), ...
            Ex(1:2:end), Ey(1:2:end), Ez(1:2:end)); % 勾配( 矢印) の表示
    hold off
end
